#!/bin/bash
#
# dumping the LDAP-database with slapcat into /var/backups/ldapdump.ldif
#
# $Id$
#

DUMPDIR=/var/backups/slapd
DUMPFILE=$DUMPDIR/ldapdump.ldif
TMPDUMPFILE=$DUMPDIR/ldapdump_tmp.ldif

# if no dump catalog exists, make one; else
# check that we have read/write access to it
if [ ! -d $DUMPDIR ] ; then 
    mkdir $DUMPDIR
    chmod 0700 $DUMPDIR
else
    if [ ! -r $DUMPDIR -o ! -w $DUPMDIR ]; then
	echo "No read- and/or writeaccess to $DUMPDIR, bailing out.."
	exit 1
    fi
fi

# do the LDAP-databasedump (slapcat)
if [ -x /etc/init.d/slapd -a -x /usr/sbin/slapcat ]; then
    /etc/init.d/slapd stop
    /usr/sbin/slapcat -l $TMPDUMPFILE
    SCRETVAL=$?
    /etc/init.d/slapd start
    if [ ! $SCRETVAL -eq 0 ]; then
	echo "/usr/sbin/slapcat failed, bailing out."
	exit 1
    fi
else
    echo "Either /etc/init.d/slapd or /usr/sbin/slapcat was not executable."
    echo "Bailing out.."
    exit 1
fi

# check that the LDAP-dumpfile is larger than zero, then replace
# the old dumpfile with the new one
if [ -s $TMPDUMPFILE ]; then
    if [ -f $DUMPFILE ]; then
	if [ ! -w $DUMPFILE ]; then
	    echo "We don't have write access to $DUMPFILE, bailing out.."
	    exit 1
	fi
    fi
    mv $TMPDUMPFILE $DUMPFILE
    echo "Successfully dumped the LDAP-database to $DUMPFILE."
else
    echo "The LDAP-dump was zero-sized, bailing out.."
    exit 1
fi

exit 0
